package org.divo;

//import java.nio.channels.SeekableByteChannel;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Map;
import java.util.ServiceLoader;

import javax.jcr.Node;
import javax.jcr.Property;
import javax.jcr.PropertyType;
import javax.jcr.Repository;
import javax.jcr.RepositoryException;
import javax.jcr.RepositoryFactory;
import javax.jcr.Session;
import javax.jcr.Value;
import javax.jcr.Workspace;
import javax.jcr.query.Query;
import javax.jcr.query.QueryResult;

import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.modeshape.common.collection.Problems;
import org.modeshape.jcr.ModeShapeEngine;
import org.modeshape.jcr.RepositoryConfiguration;

public class EasyTest {

	String uuid;

	@After
	public void shutdown() throws Exception {
	}

	Session session;

	Repository repository;

//	@Before
	public void init_mode282() throws Exception {
		 String configUrl =
		 EasyTest.class.getResource("/configRepository.xml")
		 .toExternalForm();
		 Map<String, String> parameters = Collections.singletonMap(
		 "org.modeshape.jcr.URL", configUrl);
		 for (RepositoryFactory factory : ServiceLoader
		 .load(RepositoryFactory.class)) {
		 repository = factory.getRepository(parameters);
		 if (repository != null)
		 break;
		 }
		 session = repository.login();
	}
	
	@Before
	public void init_mode300CR() throws Exception {
		// Create and start the engine ...
		ModeShapeEngine engine = new ModeShapeEngine();
		engine.start();

		// Load the configuration for a repository via the classloader (can also
		// use path to a file)...
		Repository repository = null;
		String repositoryName = null;
		try {
			URL url = EasyTest.class.getClassLoader().getResource(
					"my-repository-config.json");
			RepositoryConfiguration config = RepositoryConfiguration.read(url);

			// We could change the name of the repository programmatically ...
			// config = config.withName("Some Other Repository");

			// Verify the configuration for the repository ...
			Problems problems = config.validate();
			if (problems.hasErrors()) {
				System.err.println("Problems starting the engine.");
				System.err.println(problems);
				System.exit(-1);
			}

			// Deploy the repository ...
			repository = engine.deploy(config);
			repositoryName = config.getName();


		} catch (Throwable e) {
			e.printStackTrace();
			System.exit(-1);
			return;
		}

		try {
			// Get the repository
			repository = engine.getRepository(repositoryName);

			// Create a session ...
			session = repository.login("default");

			org.modeshape.jcr.api.nodetype.NodeTypeManager nodeTypeMgr = (org.modeshape.jcr.api.nodetype.NodeTypeManager) session
					.getWorkspace().getNodeTypeManager();
			// Then register the node types in one or more CND files
			// using a Java File object ...
			// or a URL that is resolvable to a CND file ...
			URL myCndUrl = EasyTest.class.getClassLoader().getResource(
					"nodetypes.cnd");
			nodeTypeMgr.registerNodeTypes(myCndUrl, true);
			
			// Get the root node ...
			Node root = session.getRootNode();
			assert root != null;

			System.out.println("Found the root node in the \""
					+ session.getWorkspace().getName() + "\" workspace");
		} catch (RepositoryException e) {
			e.printStackTrace();
		}
	}

	@Test
	public void testFind() throws Exception {
		Node node = session.getRootNode().addNode("second", "test:second");
		node.setProperty("myAttribute", "Second");
		Node node2 = session.getRootNode().addNode("test", "test:test");
		node2.setProperty("myAttribute", "Test2");
		Node node3 = session.getRootNode().addNode("test", "test:test");
		node3.setProperty("myAttribute", "Test3");
		Value v = session.getValueFactory().createValue(node, true);
		node2.setProperty("second", v, PropertyType.WEAKREFERENCE);
		node3.setProperty("second", v, PropertyType.WEAKREFERENCE);

		ArrayList<Value> refs = new ArrayList<Value>();
		refs.add(session.getValueFactory().createValue(node2, true));
		refs.add(session.getValueFactory().createValue(node3, true));

		Property pp = node.setProperty("testEntities",
				refs.toArray(new Value[refs.size()]),
				PropertyType.WEAKREFERENCE);
		// System.out.println(pp.getValues().length);
		session.save();
		session.refresh(true);		

		String stmt = "SELECT a.a AS * FROM [test:test] AS a INNER JOIN [test:second] AS second ON a.second = second.[jcr:uuid]";
		Query q = session.getWorkspace().getQueryManager()
				.createQuery(stmt, "JCR-SQL2");
		QueryResult qr = q.execute();
		Assert.assertEquals(2, qr.getNodes().getSize());
		Assert.assertEquals(2, qr.getRows().getSize());
		this.uuid = node.getIdentifier();
		Workspace workspace = session.getWorkspace();
		refs.clear();
		session.save();

		Node node2a = session.getNodeByIdentifier(uuid);
		Property p = node2a.getProperty("testEntities");
		Value[] values = p.getValues();

		Node node4 = session.getRootNode().addNode("test", "test:test");
		node4.setProperty("myAttribute", "testVal");
		Node node5 = session.getRootNode().addNode("test", "test:test");
		node5.setProperty("myAttribute", "testVal");

		ArrayList<Value> refs2 = new ArrayList<Value>();
		refs2.add(session.getValueFactory().createValue(node4, true));
		refs2.add(session.getValueFactory().createValue(node5, true));
		Value[] values2 = node2a.getProperty("testEntities").getValues();
		for (int i = 0; i < values2.length; i++) {
			refs2.add(values2[i]);
		}
		node2a.setProperty("testEntities",
				refs2.toArray(new Value[refs2.size()]),
				PropertyType.WEAKREFERENCE);

		session.refresh(true);
		session.save();
		Value v2a = session.getValueFactory().createValue(node2a, true);
		node4.setProperty("second", v2a, PropertyType.WEAKREFERENCE);
		node5.setProperty("second", v2a, PropertyType.WEAKREFERENCE);

		session.refresh(true);
		// session.save();

		session.getNodeByIdentifier(uuid);
		session.save();
		// session.logout();
		//
		// session = repository.login();
		session.refresh(true);
		String stmt2 = " SELECT a.a AS * FROM [test:test] AS a INNER JOIN [test:second] AS second ON a.second = second.[jcr:uuid]";
		q = session.getWorkspace().getQueryManager()
				.createQuery(stmt2, "JCR-SQL2");
		qr = q.execute();
		Assert.assertEquals(qr.getRows().getSize(), 4);
		// session.logout();
		session = workspace.getSession();
		node2a = session.getNodeByIdentifier(uuid);
		// p = node2a.getProperty("testEntities");
		// values = p.getValues();
		// System.out.println(values);

	}
}
